/****************************************************************************
 *
 *  Hameg DMM Digital Multimeter instrument driver include file
 *
 ****************************************************************************/

#ifndef __HMDMM_HEADER
#define __HMDMM_HEADER

#include "rscore.h"
#include "hmdmm_attributes.h"
#include "hmdmm_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define HMDMM_SIMULATION_ID_QUERY      "HAMEG,DMM,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define HMDMM_VALID_ID_RESPONSE_STRING "HAMEG"      /* Valid response for identification query */
#define HMDMM_ALTERNATIVE_VALID_ID_RESPONSE_STRING "Rohde&Schwarz,HMC8012"      /* Valid response for identification query */
#define HMDMM_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define HMDMM_OPC_TIMEOUT              10000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define HMDMM_CLASS_DRIVER_DESCRIPTION           ""
#define HMDMM_CLASS_DRIVER_PREFIX                ""
#define HMDMM_CLASS_DRIVER_VENDOR                ""
#define HMDMM_CLASS_DRIVER_REVISION              ""
#define HMDMM_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define HMDMM_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define HMDMM_SPECIFIC_DRIVER_DESCRIPTION        ""
#define HMDMM_SPECIFIC_DRIVER_PREFIX             "HMDMM"
#define HMDMM_SPECIFIC_DRIVER_LOCATOR            ""
#define HMDMM_SPECIFIC_DRIVER_VENDOR             ""
#define HMDMM_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define HMDMM_MINOR_VERSION                      7L    /* Instrument driver minor version          */
#define HMDMM_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define HMDMM_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define HMDMM_GROUP_CAPABILITIES                 ""
#define HMDMM_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define HMDMM_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define HMDMM_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define HMDMM_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define HMDMM_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define HMDMM_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define HMDMM_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define HMDMM_WARNMSG_OVER_RANGE           "Measurement overrange"

#define HMDMM_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define HMDMM_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define HMDMM_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define HMDMM_ERROR_CODES_AND_MSGS \
    {HMDMM_WARN_MEASURE_UNCALIBRATED,  HMDMM_WARNMSG_MEASURE_UNCALIBRATED},\
    {HMDMM_WARN_OVER_RANGE,            HMDMM_WARNMSG_OVER_RANGE},\
    {HMDMM_ERROR_MARKER_NOT_ENABLED,   HMDMM_ERRMSG_MARKER_NOT_ENABLED},\
    {HMDMM_ERROR_NOT_DELTA_MARKER,     HMDMM_ERRMSG_NOT_DELTA_MARKER},\
    {HMDMM_ERROR_MAX_TIME_EXCEEDED,    HMDMM_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

/*- Inherent Instrument Specific Attributes ----------------------------*/

/* User Options */

#define HMDMM_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define HMDMM_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define HMDMM_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define HMDMM_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define HMDMM_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define HMDMM_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define HMDMM_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define HMDMM_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

/* Class Driver Identification */

#define HMDMM_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define HMDMM_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define HMDMM_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define HMDMM_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define HMDMM_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define HMDMM_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

/* Driver Identification */

#define HMDMM_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define HMDMM_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

/* Driver Capabilities */

#define HMDMM_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define HMDMM_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define HMDMM_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define HMDMM_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

/* Driver Setup */

#define HMDMM_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

/* Instrument Identification */

#define HMDMM_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define HMDMM_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define HMDMM_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

/* Advanced Session Information */

#define HMDMM_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define HMDMM_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

/*- Instrument-Specific Attributes -------------------------------------*/

//#define HMDMM_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define HMDMM_ATTR_FUNCTION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define HMDMM_ATTR_ACTUAL_INSTRUMENT_CONFIGURATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define HMDMM_ATTR_DC_VOLTAGE_RANGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define HMDMM_ATTR_DC_VOLTAGE_AUTO_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define HMDMM_ATTR_DC_VOLTAGE_AUTOZERO                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define HMDMM_ATTR_DC_VOLTAGE_NULL_FUNCTION_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define HMDMM_ATTR_DC_VOLTAGE_NULL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define HMDMM_ATTR_DC_CURRENT_RANGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define HMDMM_ATTR_DC_CURRENT_AUTO_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define HMDMM_ATTR_DC_CURRENT_NULL_FUNCTION_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define HMDMM_ATTR_DC_CURRENT_NULL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define HMDMM_ATTR_AC_VOLTAGE_DETECTOR_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define HMDMM_ATTR_AC_VOLTAGE_RANGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define HMDMM_ATTR_AC_VOLTAGE_AUTO_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define HMDMM_ATTR_AC_VOLTAGE_NULL_FUNCTION_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define HMDMM_ATTR_AC_VOLTAGE_NULL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define HMDMM_ATTR_AC_CURRENT_DETECTOR_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define HMDMM_ATTR_AC_CURRENT_RANGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define HMDMM_ATTR_AC_CURRENT_AUTO_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define HMDMM_ATTR_AC_CURRENT_NULL_FUNCTION_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define HMDMM_ATTR_AC_CURRENT_NULL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define HMDMM_ATTR_SELF_TEST                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define HMDMM_ATTR_SAVE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define HMDMM_ATTR_RECALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define HMDMM_ATTR_DISPLAY_TEXT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define HMDMM_ATTR_DISPLAY_CLEAR                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define HMDMM_ATTR_BEEPER_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define HMDMM_ATTR_BEEP                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define HMDMM_ATTR_PRESET_REGISTER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_CONDITION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_ENABLE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_EVENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define HMDMM_ATTR_STATUS_REGISTER_OPERATION_CONDITION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define HMDMM_ATTR_STATUS_REGISTER_OPERATION_ENABLE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define HMDMM_ATTR_STATUS_REGISTER_OPERATION_EVENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define HMDMM_ATTR_FREQ_APERTURE_TIME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define HMDMM_ATTR_AUTO_FREQ_VOLTAGE_RANGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define HMDMM_ATTR_FREQ_VOLTAGE_RANGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define HMDMM_ATTR_FREQ_CURRENT_AUTO_RANGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 160
#define HMDMM_ATTR_FREQ_CURRENT_RANGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define HMDMM_ATTR_TWO_WIRES_RESISTANCE_RANGE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define HMDMM_ATTR_TWO_WIRES_RESISTANCE_AUTO_RANGE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define HMDMM_ATTR_TWO_WIRES_RESISTANCE_NULL_FUNCTION_STATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 118
#define HMDMM_ATTR_TWO_WIRES_RESISTANCE_NULL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 119
#define HMDMM_ATTR_FOUR_WIRES_RESISTANCE_RANGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define HMDMM_ATTR_FOUR_WIRES_RESISTANCE_AUTO_RANGE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define HMDMM_ATTR_FOUR_WIRES_RESISTANCE_NULL_FUNCTION_STATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define HMDMM_ATTR_FOUR_WIRES_RESISTANCE_NULL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define HMDMM_ATTR_TEMPERATURE_UNIT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define HMDMM_ATTR_TEMPERATURE_NULL_FUNCTION_STATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define HMDMM_ATTR_TEMPERATURE_NULL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define HMDMM_ATTR_TEMPERATURE_RTD_TYPE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define HMDMM_ATTR_TEMPERATURE_PROBE_TYPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define HMDMM_ATTR_CAPACITANCE_RANGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define HMDMM_ATTR_CAPACITANCE_AUTO_RANGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define HMDMM_ATTR_CAPACITANCE_NULL_FUNCTION_STATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define HMDMM_ATTR_CAPACITANCE_NULL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define HMDMM_ATTR_CONTINUITY_BEEPER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define HMDMM_ATTR_CONTINUITY_THRESHOLD                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define HMDMM_ATTR_DIODE_BEEPER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define HMDMM_ATTR_DIODE_THRESHOLD                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define HMDMM_ATTR_INITIATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define HMDMM_ATTR_SEND_SOFTWARE_TRIGGER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define HMDMM_ATTR_ADC_RATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define HMDMM_ATTR_TRIGGER_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define HMDMM_ATTR_TRIGGER_AUTO_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define HMDMM_ATTR_TRIGGER_THRESHOLD                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define HMDMM_ATTR_TRIGGER_INTERVAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define HMDMM_ATTR_TRIGGER_COUNT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define HMDMM_ATTR_MATH_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define HMDMM_ATTR_MATH_FUNCTION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define HMDMM_ATTR_MATH_OFFSET                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define HMDMM_ATTR_MATH_DB_REFERENCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define HMDMM_ATTR_MATH_DBM_REFERENCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define HMDMM_ATTR_MATH_DC_POWER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 167
#define HMDMM_ATTR_LOWER_LIMIT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define HMDMM_ATTR_UPPER_LIMIT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define HMDMM_ATTR_AVERAGE_MINIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define HMDMM_ATTR_AVERAGE_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define HMDMM_ATTR_AVERAGE_PEAK_TO_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define HMDMM_ATTR_AVERAGE_STANDARD_DEVIATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define HMDMM_ATTR_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define HMDMM_ATTR_AVERAGE_COUNT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define HMDMM_ATTR_AVERAGE_CLEAR                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define HMDMM_ATTR_HARDCOPY_FORMAT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define HMDMM_ATTR_HARDCOPY_SIZE_X                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define HMDMM_ATTR_HARDCOPY_SIZE_Y                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define HMDMM_ATTR_DATA_LOGGING                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 168
#define HMDMM_ATTR_DATA_LOGGING_FILE_FORMAT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define HMDMM_ATTR_DATA_LOGGING_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define HMDMM_ATTR_DATA_LOGGING_TIME                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_INTERVAL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define HMDMM_ATTR_ID_QUERY_RESPONSE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define HMDMM_ATTR_SYSTEM_ERROR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define HMDMM_ATTR_SYSTEM_NEXT_ERROR                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define HMDMM_ATTR_SYSTEM_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define HMDMM_ATTR_SYSTEM_LOCAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 106
#define HMDMM_ATTR_SYSTEM_REMOTE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define HMDMM_ATTR_SYSTEM_RW_LOCK                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define HMDMM_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 133
#define HMDMM_ATTR_LOGGING                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 156

// HmDmm_rngFunction
#define HMDMM_VAL_DC_VOLTS                                                  0
#define HMDMM_VAL_AC_VOLTS                                                  1
#define HMDMM_VAL_DC_CURRENT                                                2
#define HMDMM_VAL_AC_CURRENT                                                3
#define HMDMM_VAL_2_WIRE_RES                                                4
#define HMDMM_VAL_4_WIRE_RES                                                5
#define HMDMM_VAL_FREQUENCY                                                 6
#define HMDMM_VAL_TEMPERATURE                                               7
#define HMDMM_VAL_CAPACITANCE                                               8
#define HMDMM_VAL_CONTINUITY                                                9
#define HMDMM_VAL_DIODE                                                     10
#define HMDMM_VAL_FREQ_CURRENT                                              11

// HmDmm_rngBandwidth
#define HMDMM_VAL_DETECTOR_BW_SLOW                                          10
#define HMDMM_VAL_DETECTOR_BW_MEDIUM                                        50
#define HMDMM_VAL_DETECTOR_BW_FAST                                          400

// HmDmm_rngAuto
#define HMDMM_VAL_AUTO_OFF                                                  0
#define HMDMM_VAL_AUTO_ON                                                   1
#define HMDMM_VAL_AUTO_ONCE                                                 2

// HmDmm_rngMathFunction
#define HMDMM_VAL_MATH_NULL                                                 0
#define HMDMM_VAL_MATH_DB                                                   1
#define HMDMM_VAL_MATH_DBM                                                  2
#define HMDMM_VAL_MATH_AVER                                                 3
#define HMDMM_VAL_MATH_LIM                                                  4
#define HMDMM_VAL_MATH_POWER                                                5

// HmDmm_rngTransducerType
#define HMDMM_VAL_PROBE_RTD                                                 0
#define HMDMM_VAL_PROBE_FRTD                                                1

// HmDmm_rngTemperatureSensorType
#define HMDMM_VAL_SENSOR_PT100                                              0
#define HMDMM_VAL_SENSOR_PT500                                              1
#define HMDMM_VAL_SENSOR_PT1000                                             2

// HmDmm_rngAutoImpedance
#define HMDMM_VAL_AUTO_IMPEDANCE_LOW                                        0
#define HMDMM_VAL_AUTO_IMPEDANCE_HIGH                                       1

// HmDmm_rngOnOff
#define HMDMM_VAL_STATE_OFF                                                 0
#define HMDMM_VAL_STATE_ON                                                  1

// HmDmm_rngTemperatureUnit
#define HMDMM_VAL_TEMP_UNIT_C                                               0
#define HMDMM_VAL_TEMP_UNIT_F                                               1
#define HMDMM_VAL_TEMP_UNIT_K                                               2

// HmDmm_rngTemperatureRTDType
#define HMDMM_VAL_TEMP_RTD_PT100                                            0
#define HMDMM_VAL_TEMP_RTD_PT500                                            1
#define HMDMM_VAL_TEMP_RTD_PT1000                                           2

// HmDmm_rngTemperatureProbeType
#define HMDMM_VAL_TEMP_TRANSDUCER_PROBE_RTD                                 0
#define HMDMM_VAL_TEMP_TRANSDUCER_PROBE_FRTD                                1

// HmDmm_rngTriggerMode
#define HMDMM_VAL_TRIGGER_MANUAL                                            0
#define HMDMM_VAL_TRIGGER_SINGLE                                            1
#define HMDMM_VAL_TRIGGER_AUTO                                              2

// HmDmm_rngTriggerAutoMode
#define HMDMM_VAL_TRIGGER_AUTO_CONTINUE                                     0
#define HMDMM_VAL_TRIGGER_AUTO_ABOVE_THRESHOLD                              1
#define HMDMM_VAL_TRIGGER_AUTO_BELOW_THRESHOLD                              2

// HmDmm_rngHardcopyFormat
#define HMDMM_VAL_HARDCOPY_FORMAT_BMP                                       0
#define HMDMM_VAL_HARDCOPY_FORMAT_PNG                                       1

// HmDmm_rngADCRate
#define HMDMM_VAL_ADC_RATE_SLOW                                             0
#define HMDMM_VAL_ADC_RATE_MEDIUM                                           1
#define HMDMM_VAL_ADC_RATE_FAST                                             2

// HmDmm_rngDataLoggingFileFormat
#define HMDMM_VAL_DATA_LOG_FORMAT_CSV                                       0
#define HMDMM_VAL_DATA_LOG_FORMAT_TXT                                       1

// HmDmm_rngDataLoggingMode
#define HMDMM_VAL_DATA_LOG_MODE_UNLIMITED                                   0
#define HMDMM_VAL_DATA_LOG_MODE_COUNT                                       1
#define HMDMM_VAL_DATA_LOG_MODE_TIME                                        2

// HmDmm_rngIntExt
#define HMDMM_VAL_INT                                                       0
#define HMDMM_VAL_EXT                                                       1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define HMDMM_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define HMDMM_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define HMDMM_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define HMDMM_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define HMDMM_VAL_SAVE         0
#define HMDMM_VAL_RECALL       1

#define HMDMM_VAL_MEMORY_NVMEM  0
#define HMDMM_VAL_MEMORY_INTMEM 1

#define HMDMM_VAL_TO_REMOTE  0
#define HMDMM_VAL_TO_LOCAL   1

#define HMDMM_VAL_REG_QUES 0
#define HMDMM_VAL_REG_OPER 1

#define HMDMM_VAL_REG_NONE     0

#define HMDMM_VAL_REG_COND   0
#define HMDMM_VAL_REG_ENABLE 1
#define HMDMM_VAL_REG_EVENT  2

#define HMDMM_VAL_STORAGE_LOCATION_INT 0
#define HMDMM_VAL_STORAGE_LOCATION_EXT 1

/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC hmdmm_init(ViRsrc     resourceName,
                             ViBoolean  IDQuery,
                             ViBoolean  resetDevice,
                             ViSession* newInstrSession);

ViStatus _VI_FUNC hmdmm_InitWithOptions(ViRsrc     resourceName,
                                        ViBoolean  IDQuery,
                                        ViBoolean  resetDevice,
                                        ViString optionString,
                                        ViSession* newInstrSession);

ViStatus _VI_FUNC hmdmm_Configure(ViSession instrSession,
                                  ViInt32   function,
                                  ViBoolean autorange,
                                  ViReal64  range,
                                  ViReal64  resolution);

ViStatus _VI_FUNC hmdmm_Trigger(ViSession instrSession,
                                ViInt32   triggerMode,
                                ViInt32   autoMode,
                                ViReal64  threshold,
                                ViReal64  interval,
                                ViInt32   count);

ViStatus _VI_FUNC hmdmm_Temperature(ViSession instrSession,
                                    ViInt32   PTType,
                                    ViInt32   RTDConnec,
                                    ViReal64  resolution,
                                    ViInt32   unit);

ViStatus _VI_FUNC hmdmm_ApertureTime(ViSession instrSession,
                                     ViInt32   function,
                                     ViReal64 apertureTime);

ViStatus _VI_FUNC hmdmm_ACFilter(ViSession instrSession,
                                 ViInt32   function,
                                 ViInt32   detectorBandwidth);

ViStatus _VI_FUNC hmdmm_Autozero(ViSession instrSession,
                                 ViInt32   autozero);

ViStatus _VI_FUNC hmdmm_NullFunction(ViSession instrSession,
                                     ViInt32 measFunction,
                                     ViBoolean nullFunction,
                                     ViReal64  nullFunctionValue);

ViStatus _VI_FUNC hmdmm_BeeperMeasurementState(ViSession instrSession,
                                               ViInt32   function,
                                               ViBoolean beeper);

ViStatus _VI_FUNC hmdmm_ThresholdMeasurement(ViSession instrSession,
                                             ViInt32   function,
                                             ViReal64  threshold);

ViStatus _VI_FUNC hmdmm_MeasurementFunction(ViSession instrSession,
                                            ViInt32   function);

ViStatus _VI_FUNC hmdmm_SetRange(ViSession instrSession,
                                 ViInt32   function,
                                 ViBoolean autorange,
                                 ViReal64  range,
                                 ViReal64  resolution);

ViStatus _VI_FUNC hmdmm_TemperatureTransducerType(ViSession instrSession,
                                                  ViInt32   sensorType,
                                                  ViInt32   transducerType);

ViStatus _VI_FUNC hmdmm_WaitForTrigger(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_SoftwareTrigger(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_AverageClear(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_Beeper(ViSession instrSession,
                               ViBoolean beeperState,
                               ViBoolean beep);

ViStatus _VI_FUNC hmdmm_ADCRate(ViSession instrSession,
                                ViInt32   ADCRate);

ViStatus _VI_FUNC hmdmm_Measure(ViSession instrSession,
                                ViInt32   function,
                                ViBoolean autorange,
                                ViReal64  range,
                                ViReal64  resolution,
                                ViReal64* measValue);

ViStatus _VI_FUNC hmdmm_FetchMeasurement(ViSession instrSession,
                                         ViInt32   timeout,
                                         ViReal64* measurement);

ViStatus _VI_FUNC hmdmm_FetchSecondFunctionMeasurement(ViSession instrSession,
                                         ViInt32   timeout,
                                         ViReal64* measurement);

ViStatus _VI_FUNC hmdmm_ReadMeasurement(ViSession instrSession,
                                        ViInt32   timeOut_msec,
                                        ViReal64* measurement);

ViStatus _VI_FUNC hmdmm_ActiveMathOperation(ViSession instrSession,
                                            ViInt32   mathFunction,
                                            ViBoolean state);

ViStatus _VI_FUNC hmdmm_NullOperation(ViSession instrSession,
                                      ViReal64  nullValue);

ViStatus _VI_FUNC hmdmm_dBMeasurements(ViSession instrSession,
                                       ViReal64  dBReference);

ViStatus _VI_FUNC hmdmm_dBmMeasurements(ViSession instrSession,
                                        ViInt32   dBmReference);

ViStatus _VI_FUNC hmdmm_Average(ViSession instrSession,
                                ViReal64* average,
                                ViReal64* averageMin,
                                ViReal64* averageMax,
                                ViReal64* standardDeviation,
                                ViReal64* peakToPeak,
                                ViInt32*  averageCount);

ViStatus _VI_FUNC hmdmm_LimitTesting(ViSession instrSession,
                                     ViReal64  highLimit,
                                     ViReal64  lowLimit);

ViStatus _VI_FUNC hmdmm_TemperatureMeasurement(ViSession instrSession,
                                               ViInt32 ptType,
                                               ViInt32 rtdConnection,
                                               ViReal64* measurementValue);

ViStatus _VI_FUNC hmdmm_HardcopyScreenshotSize(ViSession instrSession,
                                               ViInt32*  width,
                                               ViInt32*  height);

ViStatus _VI_FUNC hmdmm_HardcopyData(ViSession instrSession,
                                     ViInt32   hardcopyFormat,
                                     ViString  fileName);

ViStatus _VI_FUNC hmdmm_DataLogging(ViSession instrSession,
                                    ViBoolean state);

ViStatus _VI_FUNC hmdmm_ConfigureDataLogging(ViSession instrSession,
                                             ViInt32   fileFormat,
                                             ViInt32   loggingMode,
                                             ViReal64  dataLoggingTime,
                                             ViInt32   numberOfMeasurement,
                                             ViReal64  measurementInterval);

ViStatus _VI_FUNC hmdmm_SetDataLoggingFileName(ViSession instrSession,
                                               ViString  fileName,
                                               ViInt32   storageLocation);

ViStatus _VI_FUNC hmdmm_ReadLoggingFileData(ViSession instrSession,
                                            ViString  fileName,
                                            ViInt32   storageLocation,
                                            ViInt32   bufferSize,
                                            ViChar    loggingFileData[],
                                            ViInt32   timeout_msec);

ViStatus _VI_FUNC hmdmm_DeleteLoggingFileData(ViSession instrSession,
                                              ViString  fileName,
                                              ViInt32   storageLocation);

ViStatus _VI_FUNC hmdmm_GetNumberOfLoggingValues(ViSession instrSession,
                                                 ViString  fileName,
                                                 ViInt32   storageLocation,
                                                 ViInt32*  numberOfLoggingValues);

ViStatus _VI_FUNC hmdmm_ListAllLoggingFilesData(ViSession instrSession,
                                                ViInt32   storageLocation,
                                                ViInt32   bufferSize,
                                                ViChar    loggingFiles[]);

ViStatus _VI_FUNC hmdmm_SetAttributeViInt32(ViSession instrSession,
                                            ViString channelName,
                                            ViAttr   attributeId,
                                            ViInt32   attributeValue);

ViStatus _VI_FUNC hmdmm_SetAttributeViReal64(ViSession instrSession,
                                             ViString channelName,
                                             ViAttr   attributeId,
                                             ViReal64  attributeValue);

ViStatus _VI_FUNC hmdmm_SetAttributeViString(ViSession instrSession,
                                             ViString channelName,
                                             ViAttr   attributeId,
                                             ViString  attributeValue);

ViStatus _VI_FUNC hmdmm_SetAttributeViBoolean(ViSession instrSession,
                                              ViString channelName,
                                              ViAttr   attributeId,
                                              ViBoolean attributeValue);

ViStatus _VI_FUNC hmdmm_GetAttributeViInt32(ViSession instrSession,
                                            ViString channelName,
                                            ViAttr   attributeId,
                                            ViInt32*  attributeValue);

ViStatus _VI_FUNC hmdmm_GetAttributeViReal64(ViSession instrSession,
                                             ViString channelName,
                                             ViAttr   attributeId,
                                             ViReal64* attributeValue);

ViStatus _VI_FUNC hmdmm_GetAttributeViString(ViSession instrSession,
                                             ViString channelName,
                                             ViAttr   attributeId,
                                             ViInt32 bufSize,
                                             ViChar    attributeValue[]);

ViStatus _VI_FUNC hmdmm_GetAttributeViBoolean(ViSession instrSession,
                                              ViString channelName,
                                              ViAttr   attributeId,
                                              ViBoolean* attributeValue);

ViStatus _VI_FUNC hmdmm_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr   attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[]);

ViStatus _VI_FUNC hmdmm_GetAttributeRepeatedCapabilityIdNames(
							ViSession instrSession,
							ViAttr   attributeId,
							ViString  repeatedCapabilityId,
							ViInt32   bufferSize,
							ViChar    repeatedCapabilityIdNames[]);

ViStatus _VI_FUNC hmdmm_reset(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_self_test(ViSession instrSession,
                                  ViInt16*  selfTestResult,
                                  ViChar    selfTestMessage[]);

ViStatus _VI_FUNC hmdmm_revision_query(ViSession instrSession,
                                       ViChar    driverRev[],
                                       ViChar    instrRev[]);

ViStatus _VI_FUNC hmdmm_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[]);

ViStatus _VI_FUNC hmdmm_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[]);

ViStatus _VI_FUNC hmdmm_Display(ViSession instrSession,
                                ViChar    displayMessage[],
                                ViBoolean clearMessage);

ViStatus _VI_FUNC hmdmm_RemoteSystemSettings(ViSession instrSession,
                                             ViInt32   instrumentState);

ViStatus _VI_FUNC hmdmm_StoreAndLoadInstrumentSetup(ViSession instrSession,
                                                    ViInt32   operation,
                                                    ViInt32   position);

ViStatus _VI_FUNC hmdmm_PresetRegister(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerType,
                                          ViInt32   statusRegisters,
                                          ViInt32   registerValue);

ViStatus _VI_FUNC hmdmm_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerType,
                                          ViInt32   statusRegisters,
                                          ViInt32*  registerValue);

ViStatus _VI_FUNC hmdmm_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_IDQueryResponse(ViSession instrSession,
                                        ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC hmdmm_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC);

ViStatus _VI_FUNC hmdmm_GetError(ViSession instrSession,
                                 ViStatus* code,
                                 ViInt32   bufferSize,
                                 ViChar    description[]);

ViStatus _VI_FUNC hmdmm_ClearError(ViSession instrSession);

ViStatus _VI_FUNC hmdmm_WriteInstrData(ViSession instrSession,
                                       ViChar    writeBuffer[]);

ViStatus _VI_FUNC hmdmm_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking);

ViStatus _VI_FUNC hmdmm_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout);

ViStatus _VI_FUNC hmdmm_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout);

ViStatus _VI_FUNC hmdmm_SetOPCTimeout(ViSession instrSession,
							 ViInt32   timeout);

ViStatus _VI_FUNC hmdmm_GetOPCTimeout(ViSession instrSession,
							 ViInt32*  timeout);

ViStatus _VI_FUNC hmdmm_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC hmdmm_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking);

ViStatus _VI_FUNC hmdmm_OPCSyncWriteEnable (ViSession instrSession,
                                            ViBoolean enable);

ViStatus _VI_FUNC hmdmm_OPCSyncQueryEnable (ViSession instrSession,
                                            ViBoolean enable);

ViStatus _VI_FUNC hmdmm_SetOpcSyncQueryMechanism (ViSession instrSession,
                                                  ViBoolean useESB,
                                                  ViBoolean CLSBefore,
                                                  ViBoolean useMAV);

ViStatus _VI_FUNC hmdmm_QueryViBoolean(ViSession instrSession,
                                       ViString   command,
                                       ViBoolean* value);

ViStatus _VI_FUNC hmdmm_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value);

ViStatus _VI_FUNC hmdmm_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value);

ViStatus _VI_FUNC hmdmm_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViInt32   bufferSize,
                                      ViChar    value[]);

ViStatus _VI_FUNC hmdmm_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViUInt32  timeout);

ViStatus _VI_FUNC hmdmm_QueryWithOPCSync(ViSession instrSession,
                                         ViString  writeBuffer,
                                         ViUInt32  timeout,
                                         ViInt32   bufferSize,
                                         ViChar    readBuffer[],
                                         ViUInt32* numBytesRead);

ViStatus _VI_FUNC hmdmm_close(ViSession instrSession);

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __HMDMM_HEADER */
