/*****************************************************************************
 *  HAMEG DMM instrument driver
 *  Callbacks
 *
 *  Original Release: March 2013
 *  By:
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "hmdmm.h"
#include "hmdmm_utility.h"
#include <stdlib.h>

#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter


ViStatus hmdmm_doubleQuotedString_ReadCallback(ViSession instrSession, ViConstString repCapName, RsCoreAttributePtr attr);
ViStatus hmdmm_doubleQuotedString_WriteCallback(ViSession instrSession, ViConstString repCapName, RsCoreAttributePtr attr, void* value);

/*****************************************************************************
 * Function: hmdmm_doubleQuotedInt_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes int
 *           with apostrophes
 *****************************************************************************/
ViStatus hmdmm_doubleQuotedInt_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
	return hmdmm_doubleQuotedString_WriteCallback(instrSession, repCapName, attr, value);
}

/*****************************************************************************
 * Function: hmdmm_doubleQuotedInt_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Reads int
 *           with apostrophes
 *****************************************************************************/
ViStatus hmdmm_doubleQuotedInt_ReadCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr)
{
	return hmdmm_doubleQuotedString_ReadCallback(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: hmdmm_doubleQuotedString_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Reads string
 *           returned with apostrophes
 *****************************************************************************/
ViStatus hmdmm_doubleQuotedString_ReadCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr)
{
	return RsCore_SpecialCallback_AnyQuotedParamRead(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: hmdmm_doubleQuotedString_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes string
 *           with apostrophes
 *****************************************************************************/
ViStatus hmdmm_doubleQuotedString_WriteCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr,
                                                void*              value)
{
	return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " \"", value, "\"");
}

/*****************************************************************************
 * Function: hmdmm_DoubleToInt_ReadCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus hmdmm_DoubleToInt_ReadCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr)
{
	ViStatus error = VI_SUCCESS;
	ViChar*  cmd = NULL;
	ViChar   result[RS_MAX_MESSAGE_BUF_SIZE] = "";

	if (attr->dataType != RS_VAL_INT32)
	{
		checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, "hmdmm_DoubleToInt_ReadCallback only accepts type RS_VAL_INT32"));
	}

	if (attr->command == NULL)
		return VI_SUCCESS;

	checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

	checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, result));
	*((ViPInt32)attr->p2value) = RsCore_Convert_String_To_ViInt32(result);

Error:
	if (cmd)
		free(cmd);

	return error;
}

/*****************************************************************************
 * Function: hmdmm_logging_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus hmdmm_logging_ReadCallback(ViSession          instrSession,
                                    ViConstString      repCapName,
                                    RsCoreAttributePtr attr)
{
	return VI_SUCCESS;
}

/*****************************************************************************
 * Function: hmdmm_logging_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus hmdmm_logging_WriteCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr,
                                     void*              value)
{
	return VI_SUCCESS;
}

